// LED Poles for Flaming Popes II: The Raftening

// NEOPIXEL BEST PRACTICES for most reliable operation:
// - Add 1000 uF CAPACITOR between NeoPixel strip's + and - connections.
// - MINIMIZE WIRING LENGTH between microcontroller board and first pixel.
// - NeoPixel strip's DATA-IN should pass through a 300-500 OHM RESISTOR.
// - AVOID connecting NeoPixels on a LIVE CIRCUIT. If you must, ALWAYS
//   connect GROUND (-) first, then +, then data.
// - When using a 3.3V microcontroller with a 5V-powered NeoPixel strip,
//   a LOGIC-LEVEL CONVERTER on the data line is STRONGLY RECOMMENDED.
// (Skipping these may work OK on your workbench but can fail in the field)

#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
 #include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif

// Which pin on the Arduino is connected to the NeoPixels?
// On a Trinket or Gemma we suggest changing this to 1:
#define LED_PIN    3

// How many NeoPixels are attached to the Arduino?
#define LED_COUNT 300

// Declare our NeoPixel strip object:
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);
// Argument 1 = Number of pixels in NeoPixel strip
// Argument 2 = Arduino pin number (most are valid)
// Argument 3 = Pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
//   NEO_RGBW    Pixels are wired for RGBW bitstream (NeoPixel RGBW products)

unsigned long startupDelay = 300000;  // how long before the animations start (milliseconds)
unsigned long performanceDuration = 420000;
// 240000 = 4 minutes
// 300000 = 5 minutes
// 360000 = 6 minutes
// 420000 = 7 minutes

// Here is where you can put in your favorite colors that will appear!
// just add new {nnn, nnn, nnn}, lines.
//                                  R   G   B
uint8_t myFavoriteColors[][3] = {{100,   100, 100},   // white
                                 {255,   0,   0},   // red 
                                 {255, 255, 255},   // white
                               };

#define FAVCOLORS sizeof(myFavoriteColors) / 3

// setup() function -- runs once at startup --------------------------------

void setup() {
  // These lines are specifically to support the Adafruit Trinket 5V 16 MHz.
  // Any other board, you can remove this part (but no harm leaving it):
#if defined(__AVR_ATtiny85__) && (F_CPU == 16000000)
  clock_prescale_set(clock_div_1);
#endif
  // END of Trinket-specific code.

  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(50); // Set BRIGHTNESS to about 1/5 (max = 255)

  randomSeed(analogRead(0)); // Seed the Random function for more truely random results
}


// loop() function -- runs repeatedly as long as board is on ---------------

void loop() {
  unsigned long currentMillis = millis();                     //  Update current time
  
  if(currentMillis >= startupDelay) { // if startupDelay has elapsed, start animation
    startupDelay = 0; //makes sure if Millis overflows, the startup delay doesn't happen again


    flashRandom(30, 10);
    flashRandom(20, 15);
    flashRandom(10, 20);  // first number is 'wait' delay, smaller num == shorter twinkle
                        // second number is how many flashes to make at this rate
    flashRandom(5, 35);
    flashRandom(3, 35);
    flashRandom(1, 35);
    
    flashAllRed(random(50));
    flashAllRed(random(50,150));
    flashAllRed(random(50));
    flashAllRed(random(50));
    flashAllRed(random(50, 150));
    flashAllRed(random(50));
    flashAllRed(random(50));
    flashAllRed(random(50,150));
    flashAllRed(random(50));
    flashAllRed(random(50));
    flashAllRed(random(50, 150));
    flashAllRed(random(50));

    theaterChase(strip.Color(255,   0,   0), 60); // Red
    theaterChase(strip.Color(255,   25,   25), random(25, 60));
    theaterChase(strip.Color(255,   50,   50), random(25, 60));
    theaterChase(strip.Color(255,   100,   100), random(25, 60));
    theaterChase(strip.Color(255,   150,   150), random(25, 60));
    theaterChase(strip.Color(255,   200,   200), random(25, 60));
    theaterChase(strip.Color(255,   255,   255), 60); // white
    theaterChase(strip.Color(255,   200,   200), random(25, 60));
    theaterChase(strip.Color(255,   150,   150), random(25, 60));
    theaterChase(strip.Color(255,   100,   100), random(25, 60));
    theaterChase(strip.Color(255,   50,   50), random(25, 60));
    theaterChase(strip.Color(255,   25,   25), random(25, 60));
    theaterChase(strip.Color(255,   0,   0), 60); // Red

    flashAllRed(random(50));
    flashAllRed(random(50, 150));
    flashAllRed(random(50));
    flashAllRed(random(50));
    flashAllRed(random(50, 150));
    flashAllRed(random(50));

while(currentMillis >= startupDelay && currentMillis < performanceDuration){
    flame();
    }
    if(currentMillis >= performanceDuration) {
      colorAll(strip.Color(0,   0,   0));
    }

  } else{
    startUpCountDown(strip.Color(50, 0, 0));
  }
}

void startUpCountDown(uint32_t color){
  int howMany = ((startupDelay-millis())/60000)+1;
  for(int i=0; i<howMany; i++) { // For each pixel in strip...
    strip.setPixelColor(i, color);         //  Set pixel's color (in RAM)
  }
  strip.show();                          //  Update strip to match
    delay(1000);                           //  Pause for a moment
  for(int i=0; i<howMany; i++) { // For each pixel in strip...
    strip.setPixelColor(i, 0);         //  Set pixel's color (in RAM)
  }
  strip.show();                          //  Update strip to match
    delay(1000);                           //  Pause for a moment
}

void flashRandom(int wait, uint8_t howmany) {

  for(uint16_t i=0; i<howmany; i++) {
    // pick a random favorite color!
    int c = random(FAVCOLORS);
    int red = myFavoriteColors[c][0];
    int green = myFavoriteColors[c][1];
    int blue = myFavoriteColors[c][2]; 

    // get a random pixel from the list
    int j = random(strip.numPixels());
    //Serial.print("Lighting up "); Serial.println(j); 
    
    // now we will 'fade' it in 5 steps
    for (int x=0; x < 5; x++) {
      int r = red * (x+1); r /= 5;
      int g = green * (x+1); g /= 5;
      int b = blue * (x+1); b /= 5;
      
      strip.setPixelColor(j, strip.Color(r, g, b));
      strip.show();
      delay(wait);
    }
    // & fade out in 5 steps
    for (int x=5; x >= 0; x--) {
      int r = red * x; r /= 5;
      int g = green * x; g /= 5;
      int b = blue * x; b /= 5;
      
      strip.setPixelColor(j, strip.Color(r, g, b));
      strip.show();
      delay(wait);
    }
  }
  // LEDs will be off when done (they are faded to 0)
}

void flashAllRed(int wait) {
  int red = 255;
    int green = 0;
    int blue = 0;

    for (int x=0; x < 5; x++) { // now we will 'fade' it in 5 steps
      int r = red * (x+1); r /= 5;
      int g = green * (x+1); g /= 5;
      int b = blue * (x+1); b /= 5;
      for(int i=0; i<strip.numPixels(); i++) { // For each pixel in strip...     
        strip.setPixelColor(i, strip.Color(r, g, b));
      }
    strip.show();                          //  Update strip to match
    delay(wait);                           //  Pause for a moment
  }
  // & fade out in 5 steps
    for (int x=5; x >= 0; x--) {
      int r = red * x; r /= 5;
      int g = green * x; g /= 5;
      int b = blue * x; b /= 5;
      for(int i=0; i<strip.numPixels(); i++) { // For each pixel in strip...     
        strip.setPixelColor(i, strip.Color(r, g, b));
      }
    strip.show();                          //  Update strip to match
    delay(wait);                           //  Pause for a moment
}
}
void flame(){
//Neopixel Flames by mysecretstache
//  Uncomment one of these RGB (Red, Green, Blue) values to
  //  set the base color of the flame.  The color will flickr
  //  based on the initial base color
  
  //  Regular (orange) flame:
    int r = 226, g = 50, b = 0;

  //  Purple flame:
  //  int r = 158, g = 8, b = 148;

  //  Green flame:
  //int r = 74, g = 150, b = 12;

  //  Flicker, based on our initial RGB values
  for(int i=0; i<strip.numPixels(); i++) {
    int flicker = random(0,55);
    int r1 = r-flicker;
    int g1 = g-flicker;
    int b1 = b-flicker;
    if(g1<0) g1=0;
    if(r1<0) r1=0;
    if(b1<0) b1=0;
    strip.setPixelColor(i,r1,g1, b1);
  }
  strip.show();

  //  Adjust the delay here, if you'd like.  Right now, it randomizes the 
  //  color switch delay to give a sense of realism
  delay(random(10,113));
}

void colorAll(uint32_t color) {
  for(int i=0; i<strip.numPixels(); i++) { // For each pixel in strip...
    strip.setPixelColor(i, color);         //  Set pixel's color (in RAM)
  }
  strip.show();                          //  Update strip to match
}

// Some functions of our own for creating animated effects -----------------

// Fill strip pixels one after another with a color. Strip is NOT cleared
// first; anything there will be covered pixel by pixel. Pass in color
// (as a single 'packed' 32-bit value, which you can get by calling
// strip.Color(red, green, blue) as shown in the loop() function above),
// and a delay time (in milliseconds) between pixels.
void colorWipe(uint32_t color, int wait) {
  for(int i=0; i<strip.numPixels(); i++) { // For each pixel in strip...
    strip.setPixelColor(i, color);         //  Set pixel's color (in RAM)
    strip.show();                          //  Update strip to match
    delay(wait);                           //  Pause for a moment
  }
}

// Theater-marquee-style chasing lights. Pass in a color (32-bit value,
// a la strip.Color(r,g,b) as mentioned above), and a delay time (in ms)
// between frames.
void theaterChase(uint32_t color, int wait) {
  for(int a=0; a<10; a++) {  // Repeat 10 times...
    for(int b=0; b<3; b++) { //  'b' counts from 0 to 2...
      strip.clear();         //   Set all pixels in RAM to 0 (off)
      // 'c' counts up from 'b' to end of strip in steps of 3...
      for(int c=b; c<strip.numPixels(); c += 3) {
        strip.setPixelColor(c, color); // Set pixel 'c' to value 'color'
      }
      strip.show(); // Update strip with new contents
      delay(wait);  // Pause for a moment
    }
  }
}

// Rainbow cycle along whole strip. Pass delay time (in ms) between frames.
void rainbow(int wait) {
  // Hue of first pixel runs 5 complete loops through the color wheel.
  // Color wheel has a range of 65536 but it's OK if we roll over, so
  // just count from 0 to 5*65536. Adding 256 to firstPixelHue each time
  // means we'll make 5*65536/256 = 1280 passes through this loop:
  for(long firstPixelHue = 0; firstPixelHue < 5*65536; firstPixelHue += 256) {
    // strip.rainbow() can take a single argument (first pixel hue) or
    // optionally a few extras: number of rainbow repetitions (default 1),
    // saturation and value (brightness) (both 0-255, similar to the
    // ColorHSV() function, default 255), and a true/false flag for whether
    // to apply gamma correction to provide 'truer' colors (default true).
    strip.rainbow(firstPixelHue);
    // Above line is equivalent to:
    // strip.rainbow(firstPixelHue, 1, 255, 255, true);
    strip.show(); // Update strip with new contents
    delay(wait);  // Pause for a moment
  }
}

// Rainbow-enhanced theater marquee. Pass delay time (in ms) between frames.
void theaterChaseRainbow(int wait) {
  int firstPixelHue = 0;     // First pixel starts at red (hue 0)
  for(int a=0; a<30; a++) {  // Repeat 30 times...
    for(int b=0; b<3; b++) { //  'b' counts from 0 to 2...
      strip.clear();         //   Set all pixels in RAM to 0 (off)
      // 'c' counts up from 'b' to end of strip in increments of 3...
      for(int c=b; c<strip.numPixels(); c += 3) {
        // hue of pixel 'c' is offset by an amount to make one full
        // revolution of the color wheel (range 65536) along the length
        // of the strip (strip.numPixels() steps):
        int      hue   = firstPixelHue + c * 65536L / strip.numPixels();
        uint32_t color = strip.gamma32(strip.ColorHSV(hue)); // hue -> RGB
        strip.setPixelColor(c, color); // Set pixel 'c' to value 'color'
      }
      strip.show();                // Update strip with new contents
      delay(wait);                 // Pause for a moment
      firstPixelHue += 65536 / 90; // One cycle of color wheel over 90 frames
    }
  }
}
